@@ -4,7 +4,7 @@
 #'
 #' @param x a numeric vector, matrix or data frame.
 #' @param y \code{NULL} (default) or a numeric vector with compatible dimsensions to \code{x}.
-#' @param factor.2.dummy logical; \code{TRUE} (default) Automatically augments variable matrix with numerical dummy variables based on the levels of factors.  Includes dependent variable \code{y}.
+#' @param factor.2.dummy logical; \code{FALSE} (default) Automatically augments variable matrix with numerical dummy variables based on the levels of factors.  Includes dependent variable \code{y}.
 #' @param tau options: ("cs", "ts", integer); 0 (default) Number of lagged observations to consider (for time series data).  Otherwise, set \code{(tau = "cs")} for cross-sectional data.  \code{(tau = "ts")} automatically selects the lag of the time series data, while \code{(tau = [integer])} specifies a time series lag.
 #' @param plot logical; \code{FALSE} (default) Plots the raw variables, tau normalized, and cross-normalized variables.
 #' @return Returns the directional causation (x ---> y) or (y ---> x) and net quantity of association.  For causal matrix, directional causation is returned as ([column variable] ---> [row variable]).  Negative numbers represent causal direction attributed to [row variable].
@@ -22,15 +22,18 @@
 #' ## AirPassengers
 #' NNS.caus(1:length(AirPassengers), AirPassengers, tau = "ts")
 #'
-#' ## Causal matrix with per factor causation
+#' ## Causal matrix without per factor causation
 #' NNS.caus(iris, tau = 0)
+#'
+#' ## Causal matrix with per factor causation
+#' NNS.caus(iris, factor.2.dummy = TRUE, tau = 0)
 #' }
 #' @export
 
 NNS.caus <- function(x, y,
-                     factor.2.dummy = TRUE,
+                     factor.2.dummy = FALSE,
                      tau = 0,
-                     plot=FALSE){
+                     plot = FALSE){
 
   orig.tau <- tau
   orig.plot <- plot
@@ -38,13 +41,13 @@ NNS.caus <- function(x, y,
   if(factor.2.dummy){
       if(!is.null(dim(x))){
           if(!is.numeric(x)){
-              x <- sapply(x,factor_2_dummy_FR)
+              x <- do.call(cbind, lapply(data.frame(x), factor_2_dummy_FR))
           } else {
-              x <- apply(x,2,as.double)
+              x <- apply(x, 2, as.double)
           }
           if(is.list(x)){
-              x <- do.call(cbind,x)
-              x <- apply(x,2,as.double)
+              x <- do.call(cbind, x)
+              x <- apply(x, 2, as.double)
           }
 
       } else {
@@ -52,7 +55,7 @@ NNS.caus <- function(x, y,
           if(is.null(dim(x))){
               x <- as.double(x)
           } else {
-              x <- apply(x,2,as.double)
+              x <- apply(x, 2, as.double)
           }
       }
   }
@@ -62,8 +65,8 @@ NNS.caus <- function(x, y,
           y <- as.numeric(y)
       }
       if(is.numeric(tau)){
-          Causation.x.given.y <- Uni.caus(x,y,tau=tau,plot = FALSE)
-          Causation.y.given.x <- Uni.caus(y,x,tau=tau,plot = FALSE)
+          Causation.x.given.y <- Uni.caus(x, y, tau = tau, plot = FALSE)
+          Causation.y.given.x <- Uni.caus(y, x, tau = tau, plot = FALSE)
 
       if(Causation.x.given.y == Causation.y.given.x |
          Causation.x.given.y == 0 | Causation.y.given.x == 0){
@@ -90,38 +93,73 @@ NNS.caus <- function(x, y,
     }
 
 
-    if(abs(Causation.x.given.y) <= abs(Causation.y.given.x)){
+    if(sign(Causation.x.given.y) == sign(Causation.y.given.x)){
+        if(abs(Causation.x.given.y) <= abs(Causation.y.given.x)){
+            if(plot){
+                # For plotting only
+                if(tau == "cs"){
+                    tau <- 0
+                }
+                if(tau == "ts"){
+                    tau <- 3
+                }
+                Uni.caus(x, y, tau = tau, plot = plot)
+            }
+            return(c(Causation.x.given.y = Causation.x.given.y,
+                  Causation.y.given.x = Causation.y.given.x,
+                  "C(x--->y)" =  abs(Causation.y.given.x) - abs(Causation.x.given.y)))
+        } else {
+          if(plot){
+              # For plotting only
+              if(tau == "cs"){
+                  tau <- 0
+              }
+              if(tau == "ts"){
+                  tau <- 3
+              }
+              Uni.caus(x, y, tau = tau, plot = plot)
+          }
+            return(c(Causation.x.given.y = Causation.x.given.y,
+                      Causation.y.given.x = Causation.y.given.x,
+                      "C(y--->x)" = abs(Causation.x.given.y) - abs(Causation.y.given.x)))
+          }
+    } else {
+      if(Causation.x.given.y <= Causation.y.given.x){
         if(plot){
-            # For plotting only
-            if(tau == "cs"){
-                tau <- 0
-            }
-            if(tau == "ts"){
-                tau <- 3
-            }
-            Uni.caus(x, y, tau = tau, plot = plot)
+          # For plotting only
+          if(tau == "cs"){
+            tau <- 0
+          }
+          if(tau == "ts"){
+            tau <- 3
+          }
+          Uni.caus(x, y, tau = tau, plot = plot)
         }
         return(c(Causation.x.given.y = Causation.x.given.y,
-               Causation.y.given.x = Causation.y.given.x,
-               "C(x--->y)" =  abs(Causation.y.given.x - Causation.x.given.y)))
-    } else {
+                 Causation.y.given.x = Causation.y.given.x,
+                 "C(x--->y)" =  Causation.y.given.x - Causation.x.given.y))
+      } else {
         if(plot){
-            # For plotting only
-            if(tau == "cs"){
-                tau <- 0
-            }
-            if(tau == "ts"){
-                tau <- 3
-            }
-            Uni.caus(x, y, tau = tau, plot = plot)
+          # For plotting only
+          if(tau == "cs"){
+            tau <- 0
+          }
+          if(tau == "ts"){
+            tau <- 3
+          }
+          Uni.caus(x, y, tau = tau, plot = plot)
         }
-    return(c(Causation.x.given.y = Causation.x.given.y,
-               Causation.y.given.x = Causation.y.given.x,
-               "C(y--->x)" = abs(Causation.x.given.y - Causation.y.given.x)))
+        return(c(Causation.x.given.y = Causation.x.given.y,
+                 Causation.y.given.x = Causation.y.given.x,
+                 "C(y--->x)" = Causation.x.given.y - Causation.y.given.x))
+      }
+
+
     }
+
   } else {
 
-    NNS.caus.matrix(x, tau = orig.tau, plot = orig.plot)
+    NNS.caus.matrix(x, tau = orig.tau)
   }
 
 
