@@ -5,9 +5,11 @@
 #' @param IVs.train a vector, matrix or data frame of variables of numeric or factor data types.
 #' @param DV.train a numeric or factor vector with compatible dimsensions to \code{(IVs.train)}.
 #' @param IVs.test a vector, matrix or data frame of variables of numeric or factor data types with compatible dimsensions to \code{(IVs.train)}.
+#' @param type \code{NULL} (default).  To perform a classification, set to \code{(type = "CLASS")}.
 #' @param obj.fn expression; \code{expression(sum((predicted - actual)^2))} (default) Sum of squared errors is the default objective function.  Any \code{expression()} using the specific terms \code{predicted} and \code{actual} can be used.
 #' @param objective options: ("min", "max") \code{"min"} (default) Select whether to minimize or maximize the objective function \code{obj.fn}.
 #' @param CV.size numeric [0, 1]; \code{NULL} (default) Sets the cross-validation size if \code{(IVs.test = NULL)}.  Defaults to 0.25 for a 25 percent random sampling of the training set under \code{(CV.size = NULL)}.
+#' @param ts.test integer; NULL (default) Sets the length of the test set for time-series data; typically \code{2*h} parameter value from \link{NNS.ARMA} or double known periods to forecast.
 #' @param folds integer; \code{folds = 5} (default) Select the number of cross-validation folds.
 #' @param order integer; \code{NULL} (default) Sets the order for \link{NNS.reg}, where \code{(order = "max")} is the k-nearest neighbors equivalent.
 #' @param norm options: ("std", "NNS", NULL); \code{NULL} (default) 3 settings offered: \code{NULL}, \code{"std"}, and \code{"NNS"}.  Selects the \code{norm} parameter in \link{NNS.reg}.
@@ -42,7 +44,7 @@
 #' @examples
 #'  ## Using 'iris' dataset where test set [IVs.test] is 'iris' rows 141:150.
 #'  \dontrun{
-#'  NNS.stack(iris[1:140, 1:4], iris[1:140, 5], IVs.test = iris[141:150, 1:4])
+#'  NNS.stack(iris[1:140, 1:4], iris[1:140, 5], IVs.test = iris[141:150, 1:4], type = "CLASS")
 #'
 #'  ## Using 'iris' dataset to determine [n.best] and [threshold] with no test set.
 #'  NNS.stack(iris[ , 1:4], iris[ , 5])
@@ -55,9 +57,11 @@
 NNS.stack <- function(IVs.train,
                       DV.train,
                       IVs.test = NULL,
+                      type = NULL,
                       obj.fn = expression( sum((predicted - actual)^2) ),
                       objective = "min",
                       CV.size = NULL,
+                      ts.test = NULL,
                       folds = 5,
                       order = NULL,
                       norm = NULL,
@@ -67,7 +71,6 @@ NNS.stack <- function(IVs.train,
                       ncores = NULL){
 
   if(is.null(obj.fn)){ stop("Please provide an objective function")}
-  objective <- tolower(objective)
 
   objective <- tolower(objective)
 
@@ -101,7 +104,7 @@ NNS.stack <- function(IVs.train,
 
   for(b in 1 : folds){
       if(status){
-          message("Folds Remaining = " ,folds-b," ","\r",appendLF=TRUE)
+          message("Folds Remaining = " , folds-b," ","\r",appendLF=TRUE)
       }
 
       set.seed(123 * b)
@@ -109,11 +112,18 @@ NNS.stack <- function(IVs.train,
 
       if(b > 1){
           test.set_half <- unique(c(rbind(test.set.1,test.set.2)))[1:(length(test.set)/2)]
+
           test.set <- unique(c(test.set_half,sample(1 : l, replace = FALSE)))[1:length(test.set)]
-    }
+          test.set <- na.omit(test.set)
+      }
+
+      if(!is.null(ts.test)){
+        test.set <- length(DV.train) - ts.test:0
+      }
 
       CV.IVs.train <- IVs.train[c(-test.set), ]
-      CV.IVs.test <- IVs.train[c(test.set), ]
+      CV.IVs.test <- do.call(cbind, lapply(data.frame(IVs.train[c(test.set), ]), factor_2_dummy_FR))
+
       CV.DV.train <- DV.train[c(-test.set)]
       CV.DV.test <- DV.train[c(test.set)]
 
@@ -134,11 +144,11 @@ NNS.stack <- function(IVs.train,
               }
 
           if(i==1){
-              setup <- NNS.reg(CV.IVs.train, CV.DV.train, point.est = CV.IVs.test, plot = FALSE, residual.plot = FALSE, n.best = i, order = order, ncores = ncores)
-
+              setup <- NNS.reg(CV.IVs.train, CV.DV.train, point.est = CV.IVs.test, plot = FALSE, residual.plot = FALSE, n.best = i, order = order, ncores = ncores, type = type, factor.2.dummy = TRUE)
               predicted <- setup$Point.est
           } else {
               predicted <- list()
+
               predicted <- foreach(j = 1:nrow(CV.IVs.test),.packages=c("NNS","data.table"))%dopar%{
                   NNS.distance(setup$RPM, dist.estimate = as.vector(CV.IVs.test[j,]), type = "L2", k = i)
               }
@@ -157,7 +167,7 @@ NNS.stack <- function(IVs.train,
           if(length(predicted > 0)){
               test.set.1 <- test.set[rev(order(abs(predicted - actual)))]
           } else {
-          test.set.1 <- test.set
+              test.set.1 <- test.set
           }
 
           if(objective=='min'){
@@ -176,15 +186,16 @@ NNS.stack <- function(IVs.train,
 
           if(b==folds){
               best.nns.cv <- mean(na.omit(unlist(best.nns.cv)))
-
               best.k <- round(fivenum(as.numeric(rep(names(table(unlist(best.k))), table(unlist(best.k)))))[4])
-
-              nns.method.1 <- NNS.reg(IVs.train, DV.train, point.est = IVs.test, plot = FALSE, residual.plot = FALSE, n.best = best.k, order = order, ncores = ncores)$Point.est
-
+              nns.method.1 <- NNS.reg(IVs.train, DV.train, point.est = IVs.test, plot = FALSE, n.best = best.k, order = order, ncores = ncores, type = type)$Point.est
+              if(!is.null(type) & !is.null(nns.method.1)){
+                  nns.method.1 <- round(nns.method.1)
+              }
               stopCluster(cl)
               registerDoSEQ()
           }
 
+
       } else {
           test.set.1 <- NULL
           best.k <- NA
@@ -198,9 +209,9 @@ NNS.stack <- function(IVs.train,
 
           actual <- CV.DV.test
 
-          var.cutoffs <- abs(round(NNS.reg(CV.IVs.train, CV.DV.train, dim.red.method = dim.red.method, plot = FALSE, residual.plot = FALSE, order=order, ncores = ncores)$equation$Coefficient, digits = 2))
+          var.cutoffs <- abs(round(NNS.reg(CV.IVs.train, CV.DV.train, dim.red.method = dim.red.method, plot = FALSE, residual.plot = FALSE, order=order, ncores = ncores, type = type)$equation$Coefficient, digits = 2))
 
-          var.cutoffs <- var.cutoffs - .005
+          var.cutoffs <- var.cutoffs - .0005
 
           var.cutoffs <- var.cutoffs[var.cutoffs <= 1 & var.cutoffs >= 0]
 
@@ -219,7 +230,7 @@ NNS.stack <- function(IVs.train,
                   message("Current NNS.reg(... , threshold = ", var.cutoffs[i] ," ) MAX Iterations Remaining = " ,length(var.cutoffs)-i," ","\r",appendLF=TRUE)
               }
 
-              predicted <- NNS.reg(CV.IVs.train, CV.DV.train, point.est = CV.IVs.test, plot = FALSE, residual.plot = FALSE, dim.red.method = dim.red.method, threshold = var.cutoffs[i], order=order, ncores = ncores)$Point.est
+              predicted <- NNS.reg(CV.IVs.train, CV.DV.train, point.est = CV.IVs.test, plot = FALSE, dim.red.method = dim.red.method, threshold = var.cutoffs[i], order = NULL, ncores = ncores, type = type)$Point.est
 
               nns.ord[i+1] <- eval(obj.fn)
 
@@ -228,13 +239,13 @@ NNS.stack <- function(IVs.train,
                   THRESHOLDS[[b]] <- best.threshold
                   best.nns.ord[[b]] <- min(na.omit(nns.ord))
                   if(i > 1 && is.na(nns.ord[i])) break
-                  if(i > 1 && (nns.ord[i] > nns.ord[i-1])) break
+                  if(i > 1 && (nns.ord[i] >= nns.ord[i-1])) break
               } else {
                   best.threshold <- var.cutoffs[which.max(na.omit(nns.ord))]
                   THRESHOLDS[[b]] <- best.threshold
                   best.nns.ord[[b]] <- max(na.omit(nns.ord))
                   if(i > 1 && is.na(nns.ord[i])) break
-                  if(i > 1 && (nns.ord[i] < nns.ord[i-1])) break
+                  if(i > 1 && (nns.ord[i] <= nns.ord[i-1])) break
               }
           }
 
@@ -243,7 +254,10 @@ NNS.stack <- function(IVs.train,
           if(b==folds){
               nns.ord.threshold <- as.numeric(names(sort(table(unlist(THRESHOLDS)),decreasing = TRUE)[1]))
               best.nns.ord <- mean(na.omit(unlist(best.nns.ord)))
-              nns.method.2 <- NNS.reg(IVs.train, DV.train,point.est = IVs.test, dim.red.method = dim.red.method, plot = FALSE, order=order, threshold = nns.ord.threshold, ncores = ncores)$Point.est
+              nns.method.2 <- NNS.reg(IVs.train, DV.train,point.est = IVs.test, dim.red.method = dim.red.method, plot = FALSE, order = order, threshold = nns.ord.threshold, ncores = ncores, type = type)$Point.est
+              if(!is.null(type) & !is.null(nns.method.2)){
+                  nns.method.2 <- round(nns.method.2)
+              }
           }
 
       } else {
@@ -274,19 +288,21 @@ NNS.stack <- function(IVs.train,
 
   if(identical(sort(method),c(1,2))){
       estimates <- (weights[1] * nns.method.1 + weights[2] * nns.method.2)
+      if(!is.null(type)){
+          estimates <- round(estimates)
+      }
   } else {
-    if(method==1){
-        estimates <- (nns.method.1)
+        if(method==1){
+            estimates <- (nns.method.1)
     } else {
         if(method==2) {
-          estimates <- (nns.method.2)
+            estimates <- (nns.method.2)
         }
     }
   }
 
   gc()
 
-
   return(list(OBJfn.reg = best.nns.cv,
               NNS.reg.n.best = best.k,
               OBJfn.dim.red = best.nns.ord,
